@@ -15,6 +15,7 @@ ________________________________________________________________________________
 #include "art.h"
 #include "empdist.h"
 #include "readSeqFile.h"
+#include "samRead.h"
 
 
 using namespace std;
@@ -25,7 +26,7 @@ double art::gaussain_sigma;
 
 int main(int argc, char* argv[]){
     cout << "=============================================================================="<<endl;
-    cout << "                            ART (Q Version 1.1.5)                             "<<endl; 
+    cout << "                            ART (Q Version 1.3.1)                             "<<endl; 
     cout << "   Copyright(c) 2008-2011, Weichun Huang, Jason Myers. All Rights Reserved.   "<<endl; 
     cout << "=============================================================================="<<endl<<endl;
 
@@ -39,6 +40,8 @@ int main(int argc, char* argv[]){
     start = clock();
 
     // Boolean flags showing the state of the run
+    bool sam_out=false;
+    bool is_matepair=false;
     bool arg_success = true;
     bool is_pairend_read=false;
     bool sep_flag = false;
@@ -59,7 +62,7 @@ int main(int argc, char* argv[]){
     string suffID=""; 
     string qual_file1 = "";
     string qual_file2 = "";
-    char* seq_file = "";
+    char* seq_file = (char*) "";
     string out_file_prefix = "";
     int read_len  = 0;
     double x_fold  = 0;
@@ -76,7 +79,15 @@ int main(int argc, char* argv[]){
     // Get Command-line arguments
     for(int i=1;i < argc;++i){
         char* arg = argv[i];
-            if(!strcmp(arg, "--paired") || !strcmp(arg, "-p") ){
+            if(!strcmp(arg, "--matepair") || !strcmp(arg, "-mp") ){
+        	num="1";
+        	is_pairend_read=true;
+        	is_matepair=true;
+	    }
+	    else if(!strcmp(arg, "--samout") || !strcmp(arg, "-sam") ){
+        	sam_out=true;
+	    }
+	    else if(!strcmp(arg, "--paired") || !strcmp(arg, "-p") ){
         	num="1";
         	is_pairend_read=true;
             } else if(!strcmp(arg, "--help") || !strcmp(arg, "-h")){
@@ -154,7 +165,7 @@ int main(int argc, char* argv[]){
 		qual_file2 = argv[i];
 		second_qual = true;
             } else if(!strcmp(arg, "--sepProf") || !strcmp(arg, "-sp")){
-		i++;
+//		i++;
 		sep_flag = true;
             } else if(!strcmp(arg, "--id") || !strcmp(arg, "-d")){
 		i++;
@@ -177,6 +188,12 @@ int main(int argc, char* argv[]){
 
     if(mean_flag && sDev_flag){
 	is_pairend_read = true;
+	if(mean>=2000){
+	       	is_matepair = true;
+	}
+	else if(is_matepair){
+		cerr<<"Warning: a mate-pair simulation may be not appropriate for DNA fragment size < 2000bp"<<endl;
+	}
 	num = "1";
     }
 
@@ -211,12 +228,17 @@ int main(int argc, char* argv[]){
 	cout << " 	art_illumina -i reference.fa -l 50 -f 10 -o single_end_data" <<endl<<endl;
 	cout << " 2) paired-end read simulation" <<endl;
 	cout << "       art_illumina -i reference.fa -p -l 50 -f 20 -m 200 -s 10 -o outFile_prefix" <<endl<<endl;
+	cout << " 3) mate-pair read simulation" <<endl;
+	cout << "       art_illumina -i reference.fa -mp -l 50 -f 20 -m 2500 -s 50 -o outFile_prefix" <<endl<<endl;
 
 	cout << " Parameters" << endl << endl;
-	cout << "  -p   --paired   indicate a paired-end read simulation" << endl;
-	cout << "  -q   --quiet    turn off end of run summary" << endl;
-	cout << "  -h   --help     print out usage information for ART" << endl;
+	cout << "  -sam --samout   indicate to generate SAM alignment file" << endl;
 	cout << "  -sp  --sepProf  indicate the use of separate quality profiles for different bases (ATGC)" << endl << endl;
+	cout << "  -mp  --matepair indicate a mate-pair read simulation" << endl;
+	cout << "  -p   --paired   indicate a paired-end read simulation" << endl;
+	cout << "       NOTE: art will automatically switch to a mate-pair read simulation if the given mean fragment size >= 2000"<<endl<<endl;
+	cout << "  -q   --quiet    turn off end of run summary" << endl;
+	cout << "  -h   --help     print out usage information for ART" << endl<<endl;
 
 	cout << "  -i   --in       the filename of input DNA reference" << endl;
 	cout << "  -o   --out      the prefix of output files" << endl;
@@ -239,10 +261,10 @@ int main(int argc, char* argv[]){
 	cout << "  of read specified for the run and were derived from re-calibrated data." << endl << endl;
 	cout << "* To simulate single-end reads one must provide the ART program with at least an input file,"<<endl;
 	cout << "   output file prefix, the read length and the fold coverage." << endl << endl;;
-	cout << "  Example: art --in reference_DNA.fa --out sim1 --len 35 --fcov 2" << endl << endl;;
+	cout << "  Example: art --in reference_DNA.fa --out sim1 --len 35 --fcov 2 -sam" << endl << endl;;
 	cout << "* To simulate paired-end reads one must use the parameters above as" << endl;
 	cout << "  well as the mean fragment length, standard deviation, and --paired tag." << endl << endl;
-	cout << "  Example: art --paired --in reference_DNA.fa --out sim2 --len 35 --fcov 2 --mflen 200 --sdev 3.5" << endl << endl;
+	cout << "  Example: art --paired --in reference_DNA.fa --out sim2 --len 35 --fcov 2 --mflen 200 --sdev 3.5 -sam" << endl << endl;
 
 	exit(0);
     }
@@ -252,11 +274,19 @@ int main(int argc, char* argv[]){
     string alnfasta=out_file_prefix+num+".aln";
     string fqfile=out_file_prefix+num+".fq";
 
+    string samfile=out_file_prefix+".sam";
+    ofstream SAMFILE;
+    if(sam_out) {
+	    SAMFILE.open(samfile.c_str(),ios::binary); 
+	    if(!SAMFILE.is_open()) { cerr<<"Can not open output file: "<<samfile<<endl; exit(0); }
+    }
+
     ofstream FQFILE(fqfile.c_str(),ios::binary);
     if(!FQFILE.is_open()) { cerr<<"Can not open output file: "<<fqfile<<endl; exit(0); }
 
     ofstream ALNFILE(alnfasta.c_str(),ios::binary);
     if(!ALNFILE.is_open()) { cerr<<"Can not open output file: "<<alnfasta<<endl; exit(0); }
+    ALNFILE<<"##ART_Illumina\tread_length\t"<<read_len<<endl;
 
     empdist::set_err_prob();
     empdist qdist(qual_file1, qual_file2, sep_flag, read_len);
@@ -351,6 +381,21 @@ int main(int argc, char* argv[]){
 		    }
 	    }
    }
+
+    samHeader sH;
+    sH.getRefseqID(seq_file);
+    sH.ID="01";
+    sH.PN="ART_Illumina"; 
+    for(int i=0;i < argc; ++i) { sH.CL.append(argv[i]);sH.CL.append(" "); }
+    sH.printAlnHeader(ALNFILE);
+
+    if(sam_out){
+	    sH.printHeader(SAMFILE);
+    }
+
+    samRead sR;
+    string srID;
+
     vector<short> qual;
     readSeqFile seq_reader(seq_file);
     string id;
@@ -368,6 +413,10 @@ int main(int argc, char* argv[]){
         string alnfasta2="";
         string fqfile2="";
     if(is_pairend_read){
+	samRead sR2; 
+	sR.rNext="=";
+	sR2.rNext="=";
+
         seqfasta2=out_file_prefix+"2.fa";
         qualfasta2=out_file_prefix+"2.qual";
         alnfasta2=out_file_prefix+"2.aln";
@@ -383,6 +432,9 @@ int main(int argc, char* argv[]){
 
         ofstream ALNFILE2(alnfasta2.c_str(),ios::binary);
         if(!ALNFILE2.is_open()) { cerr<<"Can not open output file: "<<alnfasta2<<endl; exit(0); }
+       	ALNFILE2<<"##ART_Illumina\tread_length\t"<<read_len<<endl;
+       	sH.printAlnHeader(ALNFILE2);
+
         seqRead a_read_2;
         a_read_2.set_rate(read_len,insRate2,2,a_read.ins_rate);
         a_read_2.set_rate(read_len,delRate2,2,a_read.del_rate);
@@ -407,8 +459,12 @@ int main(int argc, char* argv[]){
                 a_read.clear();
                 a_read_2.clear();
                 //a_art.next_read(a_read);
-                //a_art.next_pair_read_indel(a_read, a_read_2); 
-                a_art.next_pair_read_indel_cmp(a_read, a_read_2); 
+		if(is_matepair){
+		       	a_art.next_pair_read_indel(a_read, a_read_2); 
+		}
+		else{
+		       	a_art.next_pair_read_indel_cmp(a_read, a_read_2); 
+		}
                 if(mask_n){ 
                   if(a_read.is_plus_strand){
                     size_t bpos2=a_art.ref_seq.size()-a_read_2.bpos-read_len;
@@ -439,6 +495,7 @@ int main(int argc, char* argv[]){
 
                 a_read.add_error(qual);
                 a_read_2.add_error(qual_2);
+		srID=read_id;
                 read_id_2=read_id+"-2";
                 read_id+="-1";
 
@@ -448,15 +505,14 @@ int main(int argc, char* argv[]){
                 }
                 FQFILE<<endl;
 
-                ALNFILE<<">"<<id<<"\t"<<read_id<<"\t"<<a_read.bpos;
+                ALNFILE<<">"<<id<<"\t"<<read_id<<"-1\t"<<a_read.bpos;
                 if(a_read.is_plus_strand) ALNFILE<<"\t+\n";
                 else ALNFILE<<"\t-\n";
-                if(a_read.get_aln(aln_read,aln_ref)){
-                    ALNFILE<<aln_ref<<endl<<aln_read<<endl;
-                }
-                else{
-                    ALNFILE<<a_read.seq_ref<<endl<<a_read.seq_read<<endl;
+                if(! a_read.get_aln(aln_read,aln_ref)){
+                    aln_ref=a_read.seq_ref;
+		    aln_read=a_read.seq_read;
                 }
+                ALNFILE<<aln_ref<<endl<<aln_read<<endl;
 
                 FQFILE2<<"@"<<read_id_2<<endl<<a_read_2.seq_read<<endl<<"+"<<endl;
                 for(size_t k=0; k<qual_2.size(); k++){
@@ -467,12 +523,73 @@ int main(int argc, char* argv[]){
                 ALNFILE2<<">"<<id<<"\t"<<read_id_2<<"\t"<<a_read_2.bpos;
                 if(a_read_2.is_plus_strand) ALNFILE2<<"\t+\n";
                 else ALNFILE2<<"\t-\n";
-                if(a_read_2.get_aln(aln_read_2,aln_ref_2)){
-                    ALNFILE2<<aln_ref_2<<endl<<aln_read_2<<endl;
-                }
-                else{
-                    ALNFILE2<<a_read_2.seq_ref<<endl<<a_read_2.seq_read<<endl;
+                if(! a_read_2.get_aln(aln_read_2,aln_ref_2)){
+                    aln_ref_2=a_read_2.seq_ref;
+		    aln_read_2=a_read_2.seq_read;
                 }
+                ALNFILE2<<aln_ref_2<<endl<<aln_read_2<<endl;
+
+		if(sam_out){
+		       	sR.qname=srID;
+		       	sR.rname=id;
+
+		       	sR2.qname=srID;
+		       	sR2.rname=id;
+
+			sR.seq=a_read.seq_read;
+		       	sR.qual.resize(qual.size());
+		       	for(size_t k=0; k<qual.size(); k++){
+			       	sR.qual[k]=(char)(qual[k]+33);
+		       	}
+		
+			sR2.seq=a_read_2.seq_read;
+		       	sR2.qual.resize(qual_2.size());
+		       	for(size_t k=0; k<qual_2.size(); k++){
+			       	sR2.qual[k]=(char)(qual_2[k]+33);
+		       	}
+
+			sR.flag=0x01 | 0x02 | 0x40;
+		       	sR2.flag=0x01 | 0x02 | 0x80;
+		       	if(a_read.is_plus_strand){
+			       	sR.pos=a_read.bpos+1;
+			       	if(! is_matepair){
+				       	sR.flag =sR.flag | 0x20;
+				       	sR2.flag =sR2.flag |0x10;
+				       	sR2.pos=a_art.ref_seq.size()-(a_read_2.bpos+read_len-1);
+					sR2.reverse_comp();
+			       	}
+			       	else{
+				       	sR2.pos=a_read_2.bpos+1;
+			       	}
+		       	}
+		       	else{
+			       	sR.flag =sR.flag | 0x10;
+			       	sR.pos=a_art.ref_seq.size()-(a_read.bpos+read_len-1);
+			       	sR.reverse_comp();
+			       	if(is_matepair){
+				       	sR2.flag =sR2.flag | 0x10;
+				       	sR2.pos=a_art.ref_seq.size()-(a_read_2.bpos+read_len-1);
+				       	sR2.reverse_comp();
+				}
+				else{
+				       	sR2.pos=a_read_2.bpos+1;
+				}
+		       	}
+		       	sR.getCigar(aln_ref,aln_read);
+			sR2.getCigar(aln_ref_2,aln_read_2);
+		       	sR.pNext=sR2.pos;
+		       	sR2.pNext=sR.pos;
+		       	if(sR2.pos>sR.pos){
+			       	sR.tLen=sR2.pos+a_read_2.seq_read.size()-sR.pos;
+			       	sR2.tLen=-sR.tLen;
+		       	}
+		       	else{
+			       	sR2.tLen=sR.pos+a_read.seq_read.size()-sR2.pos;
+			       	sR.tLen=-sR2.tLen;
+		       	} 
+			sR.printRead(SAMFILE);
+			sR2.printRead(SAMFILE);
+		}
 
                 t_num_read-=2;
             }
@@ -525,12 +642,37 @@ int main(int argc, char* argv[]){
                 ALNFILE<<">"<<id<<"\t"<<read_id<<"\t"<<a_read.bpos;
                 if(a_read.is_plus_strand) ALNFILE<<"\t+\n";
                 else ALNFILE<<"\t-\n";
-                if(a_read.get_aln(aln_read,aln_ref)){
-                    ALNFILE<<aln_ref<<endl<<aln_read<<endl;
-                }
-                else{
-                    ALNFILE<<a_read.seq_ref<<endl<<a_read.seq_read<<endl;
+
+                if(! a_read.get_aln(aln_read,aln_ref)){
+                    aln_ref=a_read.seq_ref;
+		    aln_read=a_read.seq_read;
                 }
+                ALNFILE<<aln_ref<<endl<<aln_read<<endl;
+
+		if(sam_out){
+		       	sR.qname=read_id;
+		       	sR.rname=id;
+		       	sR.flag =0;
+
+			sR.seq=a_read.seq_read;
+		       	sR.qual.resize(qual.size());
+		       	for(size_t k=0; k<qual.size(); k++){
+			       	sR.qual[k]=(char)(qual[k]+33);
+		       	}
+
+		       	if(a_read.is_plus_strand){
+			       	sR.pos=a_read.bpos+1;
+		       	}
+		       	else{
+			       	sR.flag = 0x10;
+			       	sR.pos=a_art.ref_seq.size()-(a_read.bpos+a_read.seq_read.size()-1);
+			       	sR.reverse_comp();
+		       	}
+		       	sR.getCigar(aln_ref,aln_read);
+			sR.printRead(SAMFILE);
+		}
+
+
                 t_num_read--;
             }
         }
@@ -538,6 +680,9 @@ int main(int argc, char* argv[]){
 
     FQFILE.close();
     ALNFILE.close();
+    if(sam_out){
+	    SAMFILE.close();
+    }
 
     if(!is_pairend_read){
         cout << "                          Single-end Simulation" << endl << endl;
@@ -599,17 +744,21 @@ int main(int argc, char* argv[]){
 	cout << "Output files" << endl << endl;
 
 	if(is_pairend_read){
-	    cout << "  Sequence Files:" << endl; 
-	    cout << "\t the 1st reads: " << fqfile.c_str() << endl;
-	    cout << "\t the 2nd reads: " << fqfile2.c_str() << endl << endl;
-	    cout << "  Alignment Files:" << endl; 
-	    cout << "\t the 1st reads: " << alnfasta.c_str() << endl;
-	    cout << "\t the 2nd reads: " << alnfasta2.c_str() << endl << endl;;
+	    cout << "  FASTQ Sequence Files:" << endl; 
+	    cout << "\t the 1st reads: " << fqfile << endl;
+	    cout << "\t the 2nd reads: " << fqfile2 << endl << endl;
+	    cout << "  ALN Alignment Files:" << endl; 
+	    cout << "\t the 1st reads: " << alnfasta << endl;
+	    cout << "\t the 2nd reads: " << alnfasta2 << endl << endl;;
 	} else {
-	    cout << "  Sequence File:" << endl; 
-	    cout << "\t" << fqfile.c_str() << endl << endl;
-	    cout << "  Alignment File:" << endl; 
-	    cout << "\t" << alnfasta.c_str() << endl << endl;
+	    cout << "  FASTQ Sequence File:" << endl; 
+	    cout << "\t" << fqfile << endl << endl;
+	    cout << "  ALN Alignment File:" << endl; 
+	    cout << "\t" << alnfasta << endl << endl;
+	}
+	if(sam_out){
+	    cout << "  SAM Alignment File:" << endl; 
+	    cout << "\t" << samfile << endl << endl;
 	}
     }    
 
